@@ -14,6 +14,8 @@ use async_std::io::Cursor;
 extern crate biscuit_auth as biscuit;
 use biscuit::{crypto::PublicKey, token::Biscuit};
 use surf::http::{Url, Method, Mime};
+use sha2::{Sha256, Digest};
+use chrono::prelude::*;
 
 lazy_static! {
     static ref DB : Arc<rocksdb::DB> = {
@@ -73,6 +75,17 @@ lazy_static! {
         ]);
         Arc::new(col)
     };
+
+    static ref CACHE : Arc<lockfree::map::Map<String, Cache>> = {
+        let col : lockfree::map::Map<String, Cache> = lockfree::map::Map::new();
+        Arc::new(col)
+    };
+}
+
+#[derive(Debug, Serialize, Deserialize, Clone)]
+pub struct Cache {
+    pub last_modified: String,
+    pub e_tag: String,
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone)]
@@ -370,13 +383,13 @@ async fn get_url_object(req: Request<()>) -> tide::Result {
                         match provider_value {
                             Some(provider) => {
                                 let bucket_name = bucket;
-                                let region = region.parse()?;
+                                let region: s3compat::Region = region.parse()?;
                             
                                 let access_key = provider.access_key;
                                 let secret_key = provider.secret_key;
                                 let credentials = Credentials::new(Some(&access_key), Some(&secret_key), None, None, None)?;
                             
-                                let bucket = Bucket::new(&bucket_name, region, credentials)?;
+                                let bucket = Bucket::new(&bucket_name, region.clone(), credentials)?;
 
                                 let (data, code) = bucket.get_object(object_path.clone()).await?;
 
@@ -390,8 +403,31 @@ async fn get_url_object(req: Request<()>) -> tide::Result {
                                     }
                                 }
 
+                                let mut hasher = Sha256::new();
+                                hasher.update(data.clone());
+                                let etag_hash: String = format!("{:X}", hasher.finalize());
+
+                                let key = format!("{}-{}-{}", region.clone().to_string(), bucket_name, object_path);
+
+                                let mut lmd = Utc::now().format("%a, %d %b %Y %H:%M:%S GMT").to_string();
+                                for rg in CACHE.iter() {
+                                    if rg.key() == &key {
+                                        if rg.val().e_tag == etag_hash {
+                                            lmd = rg.val().clone().last_modified;
+                                        }
+                                    }
+                                }
+
+                                CACHE.insert(key, Cache{
+                                    last_modified: lmd.clone(),
+                                    e_tag: etag_hash.clone(),
+                                });
+
                                 return Ok(tide::Response::builder(code).body(data)
                                     .header("content-type", content_type)
+                                    .header("cache-control", "no-cacne")
+                                    .header("etag", etag_hash)
+                                    .header("last-modified", lmd)
                                     .build());
                             },
                             None => {
@@ -441,13 +477,13 @@ async fn get_object(mut req: Request<()>) -> tide::Result {
                         match provider_value {
                             Some(provider) => {
                                 let bucket_name = object_form.bucket;
-                                let region = object_form.region.parse()?;
+                                let region: s3compat::Region = object_form.region.parse()?;
                             
                                 let access_key = provider.access_key;
                                 let secret_key = provider.secret_key;
                                 let credentials = Credentials::new(Some(&access_key), Some(&secret_key), None, None, None)?;
                             
-                                let bucket = Bucket::new(&bucket_name, region, credentials)?;
+                                let bucket = Bucket::new(&bucket_name, region.clone(), credentials)?;
 
                                 let (data, code) = bucket.get_object(object_form.object_path.clone()).await?;
 
@@ -461,8 +497,31 @@ async fn get_object(mut req: Request<()>) -> tide::Result {
                                     }
                                 }
 
+                                let mut hasher = Sha256::new();
+                                hasher.update(data.clone());
+                                let etag_hash: String = format!("{:X}", hasher.finalize());
+
+                                let key = format!("{}-{}-{}", region.clone().to_string(), bucket_name, object_form.object_path);
+
+                                let mut lmd = Utc::now().format("%a, %d %b %Y %H:%M:%S GMT").to_string();
+                                for rg in CACHE.iter() {
+                                    if rg.key() == &key {
+                                        if rg.val().e_tag == etag_hash {
+                                            lmd = rg.val().clone().last_modified;
+                                        }
+                                    }
+                                }
+
+                                CACHE.insert(key, Cache{
+                                    last_modified: lmd.clone(),
+                                    e_tag: etag_hash.clone(),
+                                });
+
                                 return Ok(tide::Response::builder(code).body(data)
                                     .header("content-type", content_type)
+                                    .header("cache-control", "no-cacne")
+                                    .header("etag", etag_hash)
+                                    .header("last-modified", lmd)
                                     .build());
                             },
                             None => {
